.input-group {
  margin-bottom: 10px;
  .input-group-addon {
    border: 0px;
    text-align: right;
    background-color: #fff;
    font-weight: 700;
    font-size: 0.875em;
    color: #5c5c5c;
    border: none;
    border-radius: 0;
    display: inline-block;
  }
}
#content-core .select2-container .select2-choices .select2-input.select2-input:hover,
#content .select2-container .select2-choices .select2-input.select2-input:hover {
  border: 0;
}

input[type="submit"] {
  cursor: pointer;
}
