.side-nav.markers a.plone-btn {
  width: 200px;
  display: inline;
}

.pat-slider {
    .slider {
        overflow: hidden;
        width: 100%;
    }

    ul {
        position: relative;
        list-style-type: none;
        margin-left: 0px;

        li {
            display: block;
            list-style: none;
            float: left;
        }
    }
}

.template-file_view.portaltype-video .leadImage,
.template-file_view.portaltype-audio .leadImage,
.template-file_view.portaltype-file .leadImage {
    display: none;
}

.mosaic-width-half .mosaic-subscription-tile {
    color: blue;
    .list-field {
        min-width: 7.5em;
    }

    .ordered-selection-field button {
      margin-left: 0.5em;
    }
}

/* reset dateinput .date styles */
body .date {
    border: 0px;
    padding: 0px;
    text-align: left;
    box-shadow: none;
    -moz-box-shadow: none;
}

/* XXX tooltip class fixes
  Foundation tooltips are not compatible with bootstrap tooltips
*/
body .tooltip {
  width: inherit;
  max-width: inherit;
  background: inherit;
}


.hiddenStructure {
  display: none;
}


.picker__box {
  .picker__header {
    .picker__select--month,
    .picker__select--year {
      display: inline-block;
      margin: 0 6px;
      }
    .picker__select--year {
      width: 25%;
    }
    .picker__select--month {
      width: 40%;
    }
    .picker__nav--prev::before {
      border-bottom: 1.1em solid transparent;
      border-right: 1em solid #000000;
      border-top: 1.1em solid transparent;
    }
    .picker__nav--next::before {
      border-bottom: 1.1em solid transparent;
      border-left: 1em solid #000000;
      border-top: 1.1em solid transparent;
    }
  }

  .picker__footer {
    button.picker__button--today,
    button.picker__button--clear,
    button.picker__button--close {
      color: #4d4d4d;
    }
  }
}
.picker__list-item {
  font-size: 11px;
  padding: 6px 12px;
  width: 50%;
}


.DV-fullscreenContainer {
  // full screen not working right now
  display: none;
}
