
fieldset.image .form-inline {
  text-align: left;
}

.sr-only,
.hiddenStructure {
  display: none;
}

#global_statusmessage {
  margin: 10px;
}

.visualClear {
  clear: both;
}

.newsImageContainer {
  float: right;
}

// Search page paging style
.pagination > li.current > a {
  background-color: #eeeeee;
  border-color: #dddddd;
  color: #1abcee;
}
