@import (less) "@{bowerPath}/tinymce-builded/js/tinymce/skins/lightgray/skin.less";
@import (less) "@{bowerPath}/tinymce-builded/js/tinymce/skins/lightgray/Content.Objects.less";
@import (inline) "@{bowerPath}/tinymce-builded/js/tinymce/plugins/visualblocks/css/visualblocks.css";
@import "@{mockupPath}/modal/pattern.modal.less";
@import "@{mockupPath}/autotoc/pattern.autotoc.less";
@import "@{mockupPath}/upload/less/pattern.upload.less";

div.linkModal {
    .pattern-relateditems-container{
        padding-top: 0;
        margin-right: 20px;
    }
    hr {
        margin: 5px 0;
    }
    div.linkTypes {
        line-height: 15px;
        input{
            margin: 5px 5px 9px 9px;
        }
    }
    input[name="external"],input[name="externalImage"]{
        width: 80%;
    }
    fieldset{
        padding-top: 10px;
        min-height: 90px;
    }
    fieldset .input-wrapper{ display: inline-flex; }
    .common-controls{
        padding-top: 10px;
        border-top: 1px solid #ccc;
    }
}

.tinymce-dropzone{
    position: relative;

    .dz-notice {
        display: none;
        -ms-filter: none;
        filter: none;
        pointer-events: none;
    }
    * {
        opacity: 1;
    }

    .dropzone-previews{
        background-color: #f5f5f5;
    }
    .fallback{
        /* do not display fallback, just don't work for these weirdos */
        display: none;
    }
}

.tinymce-dropzone.dz-drag-hover{
    box-sizing: border-box;

    * {
        opacity: 0.7;
    }
    iframe {
        pointer-events: none;
    }
    p{
        font-size: 250%;
        position: absolute;
        width: 400px;
        height: 50px;
        top: 50%;
        left: 50%;
        margin-left: -200px;
        display: block;
        text-align: center;
        opacity: 1;
    }
    .dz-notice{
        display: block;
        opacity: 1;
        -ms-filter: none;
        filter: none;
        -webkit-transition: opacity 0.3s ease-in-out;
        -moz-transition: opacity 0.3s ease-in-out;
        -o-transition: opacity 0.3s ease-in-out;
        -ms-transition: opacity 0.3s ease-in-out;
        transition: opacity 0.3s ease-in-out;
        background-position: 0 0;
        position: absolute;
        width: 100%;
        height: 100%;
        top: 0;
        left: 0;
    }
}

.pattern-relateditems-result-image img{
    max-height: 30px;
    max-width: 80px;
}

.mce-item-table td {
	min-width: 0.6em;  // enforce a min-width while editing for tables with visual aid turned on
}
