.plone-btn-danger,
.castle-cropper-container
.plone-btn-danger {
  background-color: @castle_red_color !important;
  color: white !important;
  border: none !important;
  -webkit-border-radius: 0px !important;
  -moz-border-radius: 0px !important;
  border-radius: 0px !important;
  margin-right: 10px;
  &:hover, &:active {
    background-color: @castle_darker_color !important;
  }
}
