//PAGINATION STYLES
ul.castle-pagination{
  margin:30px 0 20px;
  list-style:none;
  text-align: center;

  li{
    display:inline-block;

    a{
      display: inline-block;
      margin-top: 5px;
      margin-right:5px;
      font-size:14px;
      color:@castle-primary;
      padding: 6px 12px;
      text-decoration: none;
      background-color: #ffffff;
      border: 1px solid #dddddd;
      border-radius:0;
      line-height: 1.42857143;

      &:hover, &:focus{
        background-color:@castle-primary;
        color:#fff;
      }
    }
  }

  .current a{
    background-color:@castle-primary;
    border-color:@castle-primary;
    color:#fff;

    &:hover, &:focus{
      background-color:darken(@castle-primary, 10%);
    }
  }
}
