@castle_red_color: #C72527;
@castle_light_grey_color: #CECECE;
@castle_dark_grey_color: #292929;
@castle_lighter_dark_grey_color: #414141;
@castle_white_color: #FFF;
@castle_dark_color: #292959;
@castle_darker_color: #292929;
@castle_very_light_blue: #F2FAFD;
@castle_off_white: #F4F4F4;
@castle_bluish_grey: #1F4E82;
@castle_lightest_grey_ever: #FBFBFB;
@castle_edit_zone_grey: #EBEBEB;
@castle_edit_zone_text: #CCCCCC;
@castle_edit_zone_hover: #F3F3F3;
@castle_new_gray: #BCBCBC;

// Brand Colors
@castle-primary:#1F4E82;
@castle-primary-dark: darken(@castle-primary, 20%);
@castle-secondary:#C72527;
@castle-secondary-dark: darken(@castle-secondary, 20%);
@castle_very_light_blue: #F2FAFD;

// Shades of Grey
@castle-base-grey: #F5F5F5;
@castle-lighter-grey: darken(@castle-base-grey, 20%);   //#c2c2c2
@castle-mid-light-grey: darken(@castle-base-grey, 30%); //#a9a9a9
@castle-light-grey: darken(@castle-base-grey, 50%);		//#767676
@castle-medium-grey: darken(@castle-base-grey, 60%);	//#5c5c5c
@castle-dark-grey: darken(@castle-base-grey, 70%);		//#434343
@castle-darker-grey: darken(@castle-base-grey, 80%);	//#292929

//Typography
@font-family-base: 'Open Sans', Arial, sans-serif;
@font-size-base: 16px;
@font-color-primary: #292929;

//Buttons

//Links

//Modals
@modal-box-color: #fff;
@modal-button: @castle-primary;
@modal-button-hover: darken(@modal-button, 10%);
@modal-text-color:#292929;
@modal-text-hover-color: lighten(@modal-text-color, 20%);
@modal-border:#e5e5e5;


// publication state colors
@published-color: #299b29; //published and external same color
@draft-color: #fab82a; // lime-yellow //draft is visible
@pending-color: #e2e721; // orange
@private-color: #c4183c; // red
@internal-color: #fab82a; // is draft
@internally-published-color: #883dfa; // is intranet


// TOOLBAR
@toolbar-color: #fff;
@toolbar-sidebar-color: #f5f5f5;
@toolbar-border-color: #dedede;
@toolbar-box-shadow-color: rgba(0,0,0,0.16);
@toolbar-text-color: #292929;
@toolbar-link-color: #007bb3;
@toolbar-link-hover-color: #1f4e82;
@toolbar-separator-color: #dedede;
@toolbar-link: @plone-link-color;

@toolbar-btn-bg-color: #ffffff;
@toolbar-btn-text-color: #292929;

@screen-xs-min: 480px;
@screen-xs-max: 767px;
@screen-sm-min: 768px;
@screen-sm-max: 991px;
@screen-md-min: 992px;
@screen-md-max: 1199px;
@screen-lg-min: 1200px;

@left-toolbar-width: 190px;
@top-toolbar-height: 55px;

//TABLES
@table-bg-accent: #F8F8F8;
