
// Upload Content Modal
#upload-content-modal {

}

.droparea {
  height: 275px;
  background: #f8f8f8;
  border: 2px dashed #dedede;
  border-radius: 0px;
  padding: 20px;
  text-align: center;

  .droptext-container {
    margin-top: 40px;
  }

  .upload-text {
    font-family: 'Open Sans', Arial, sans-serif;
    color: #c3c5c6;
    font-size: 1em;
    margin-bottom: 0.3em;
  }

  .castle-btn-select-files {
    background-color: #e8e8e8;
    padding: 5px 10px;
    margin-top: 5px;

    border-radius: 0px;
    text-shadow: none;
    color: #969696;
  }
}
.dragging .droparea {
  background-color: @castle_very_light_blue;
}


.castle-upload-container {
  background: #ffffff;
  padding: 20px;
  overflow: auto;

  .btn-container {
    padding: 15px 0px 15px 10px;
  }

  &.has-files {
    .droparea {
      height: 110px;
      padding: 10px;

      .droptext-container {
        margin: 0px;

        .upload-text {
          margin-bottom: 0px;
          line-height: 1.15em;
        }
      }
    }
  }

  .file-list {
    margin-top: 10px;

    h5{
      margin-bottom:20px;
    }

    label{
      text-align: right;
      padding-right: 5px;
    }

    label.required::after {
      content: "*";
      color: red;
    }

    > ul {
      list-style: none;
      margin: 0px;
      padding: 0px;

      li.file-container {
        border-top: 1px solid #dedede;
        padding: 10px 0;
        width: 100%;
      }
    }
    .preview-container {
      text-align: center;

      > span, img {
        margin: 1em auto;
      }

      .glyphicon-file {
        font-size: 50px;
      }
    }

    .bottom-container {
      text-align: right;
    }

    .field {
      margin-left: -15px;
      margin-right: -15px;
    }

    .castle-btn-remove, .castle-btn-upload{
      background: #fff;
      font-size: 1.2em;
      width: 1.8em;
      height: 1.8em;
      border-radius: 1em;
      text-align: center;
      padding: 5px;
      margin-left:8px;
      box-shadow: none;
      line-height: 1.25em;
    }

    .castle-btn-remove{
        .icon-remove::before{
          width:0.8em;
          height:0.8em;
          line-height: 1.3em;
          margin:0;
          vertical-align: initial;
        }

        &:hover{
          background:#C72527;
        }
      }

      .castle-btn-upload{
        .icon-ok::before{
          width:0.9em;
          height:0.9em;
          line-height: 1.3em;
          margin:0;
          vertical-align: initial;
        }

        &:hover{
          background:#299b29;
        }
      }
  }

  .castle-progress-container {
    text-align: center;
    clear: both;
    .castle-progress {
      width: 100%;
      border-top: 2px solid #dedede;
      .castle-progress-inner {
        border-top: 2px solid #575779;
        display: block;
        margin-top: -2px;
      }
    }
  }
}

.dz-error-message {
  width: 100%;
  clear: both;
  margin-top: 1em;
}

.dz-filename {
  text-overflow: ellipsis;
  overflow: hidden;
}

.castle-cropper-container {
  max-height: 1200px;

  .plone-btn-group{
    margin: 20px 0 10px;

    button{
      margin-right:8px;
    }
  }
}

.focalpoint-image-container {
  position: relative;
  width: 300px;
  cursor: pointer;
  border: 1px solid #ddd;
  box-shadow: 0px 0px 5px 2px #ddd;
  margin-right: 15px;

  .focalpoint-image-center {
    position: absolute;
    width: 24px;
    height: 24px;
    background-color: #949393;
    border-radius: 12px;
    border: 4px solid #ccc;
    opacity: 0.7;
    box-shadow: 0 0 5px 0px #A7A7A7;
  }
}

.castle-imagewidget-container {
  padding: 15px;

  .btn-container {
    padding: 15px;

    button {
      margin-right: 20px;
    }
  }

  .droparea {
    height: 100px;
  }
}


/* location picker */
.castle-upload-container {
  .pick-location, .no-location {

  }
  .pick-location{
    border: 2px dashed #dedede;
    border-top: 0px;
    width: 400px;
    position: relative;
    height: 30px;
    overflow: hidden;

    > div {
      position: absolute;
      top: 0;
      height: 30px;
    }
    .pick-location-folder {
      left: 0;
      background-color: #dedede;
      padding: 6px 0px 0px 12px;
    }
    .pick-location-location {
      left: 50px;
      line-height: 27px;
    }
    .pick-location-remove {
      right: 0;

      button.remove {
        padding: 2px 11px;
        font-size: 1.1em;
      }
    }
  }
  .no-location {
    font-size: 13px;
  }
}
