
@font-face {
  font-family: 'Open Sans';
  font-weight: 400;
  font-style: normal;
  src: url('../fonts/Open-Sans-regular/Open-Sans-regular.eot');
  src: url('../fonts/Open-Sans-regular/Open-Sans-regular.eot?#iefix') format('embedded-opentype'),
       local('Open Sans'),
       local('Open-Sans-regular'),
       url('../fonts/Open-Sans-regular/Open-Sans-regular.woff2') format('woff2'),
       url('../fonts/Open-Sans-regular/Open-Sans-regular.woff') format('woff'),
       url('../../++plone++castle/less/fonts/Open-Sans-regular/Open-Sans-regular.ttf') format('truetype'),
       url('../../++plone++castle/less/fonts/Open-Sans-regular/Open-Sans-regular.svg#OpenSans') format('svg');
}

@font-face {
  font-family: 'Open Sans 600';
  font-weight: 600;
  font-style: normal;
  src: url('../fonts/Open-Sans-600/Open-Sans-600.eot');
  src: url('../fonts/Open-Sans-600/Open-Sans-600.eot?#iefix') format('embedded-opentype'),
       local('Open Sans Semibold'),
       local('Open-Sans-600'),
       url('../fonts/Open-Sans-600/Open-Sans-600.woff2') format('woff2'),
       url('../fonts/Open-Sans-600/Open-Sans-600.woff') format('woff'),
       url('../../++plone++castle/less/fonts/Open-Sans-600/Open-Sans-600.ttf') format('truetype'),
       url('../../++plone++castle/less/fonts/Open-Sans-600/Open-Sans-600.svg#OpenSans') format('svg');
}

@font-face {
  font-family: 'Open Sans 700';
  font-weight: 700;
  font-style: normal;
  src: url('../fonts/Open-Sans-700/Open-Sans-700.eot');
  src: url('../fonts/Open-Sans-700/Open-Sans-700.eot?#iefix') format('embedded-opentype'),
       local('Open Sans Bold'),
       local('Open-Sans-700'),
       url('../fonts/Open-Sans-700/Open-Sans-700.woff2') format('woff2'),
       url('../fonts/Open-Sans-700/Open-Sans-700.woff') format('woff'),
       url('../../++plone++castle/less/fonts/Open-Sans-700/Open-Sans-700.ttf') format('truetype'),
       url('../../++plone++castle/less/fonts/Open-Sans-700/Open-Sans-700.svg#OpenSans') format('svg');
}

@font-face {
  font-family: 'Open Sans Condensed';
  font-weight: 700;
  font-style: normal;
  src: url('../fonts/Open-Sans-Condensed-700/Open-Sans-Condensed-700.eot');
  src: url('../fonts/Open-Sans-Condensed-700/Open-Sans-Condensed-700.eot?#iefix') format('embedded-opentype'),
       local('Open Sans Condensed Bold'),
       local('Open-Sans-Condensed-700'),
       url('../fonts/Open-Sans-Condensed-700/Open-Sans-Condensed-700.woff2') format('woff2'),
       url('../fonts/Open-Sans-Condensed-700/Open-Sans-Condensed-700.woff') format('woff'),
       url('../../++plone++castle/less/fonts/Open-Sans-Condensed-700/Open-Sans-Condensed-700.ttf') format('truetype'),
       url('../../++plone++castle/less/fonts/Open-Sans-Condensed-700/Open-Sans-Condensed-700.svg#OpenSansCondensed') format('svg');
}
