#content-core .search-form .search-input-group,.search-form .search-input-group{width:80%;position:relative;border:1px solid #d9d9d9;margin-left:15px;height:36px}#content-core .search-form .search-input,.search-form .search-input{top:0;border:1px solid transparent;padding:0;margin:0;padding-left:8px;width:calc(100% -  100px );position:absolute;z-index:6;left:0;outline:none;background:url("data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw%3D%3D") transparent;font:24px arial,sans-serif;line-height:34px !important;height:34px !important}#content-core .search-form .search-btn,.search-form .search-btn{position:absolute;top:0;right:0;width:100px;background-color:#4285f4;color:white;border:none;border-left:1px solid #d9d9d9;height:34px}.search-options{margin-top:20px;margin-bottom:10px;margin-left:-10px;margin-right:-10px;padding-top:20px;border-top:1px solid #d9d9d9;border-bottom:1px solid #d9d9d9}.search-options>ul{position:relative;padding:0;margin:0;margin-left:30px;display:inline-block}.search-options>ul>li{display:inline-block;height:35px;margin-right:20px;padding:0 7px}.search-options>ul>li>a{display:block;color:#777777}.search-options>ul>li>a:hover{color:#403f3f;text-decoration:none}.search-options>ul>li.active{border-bottom:3px solid #4285f4}.search-options>ul>li.active>a,.search-options>ul>li.active>a:hover{font-weight:bold;color:#4285f4}.search-menu-dropdown-container{position:relative}.search-menu-dropdown-container a.more-btn{position:relative}.search-menu-dropdown-container a.more-btn::after{content:" ";border-color:#909090 transparent;border-style:solid;border-width:4px 4px 0 4px;width:0;height:0;margin-left:5px;top:50%;margin-top:-1px;position:absolute}.search-menu-dropdown-container .search-menu-dropdown{position:absolute;top:25px;right:0;z-index:5;background-color:white;border:1px solid #d9d9d9;box-shadow:0 0 5px 0 #ccc;-webkit-box-shadow:0 0 5px 0 #ccc;margin:0;padding:5px 0;min-width:150px}.search-menu-dropdown-container .search-menu-dropdown li{list-style:none}.search-menu-dropdown-container .search-menu-dropdown li a{display:block;padding:2px 20px;color:#777777}.search-menu-dropdown-container .search-menu-dropdown li a:hover{background-color:#e8e8e8;text-decoration:none}.search-additional-sites{float:right}@media (max-width:768px){.search-additional-sites{position:relative}}.search-suggestions ul{list-style:none;margin:0}.search-suggestions ul li{display:inline-block}.search-suggestions .label{padding-right:5px;color:#8e8e8e}.search-suggestions .suggestion{padding-right:3px}.search-suggestions .suggestion::after{content:", "}.search-suggestions .suggestion:last-child::after{content:""}#search-results-wrapper{margin-left:15px}#search-results-bar{color:#716f6f;font-size:90%}#search-results{margin-top:10px}#search-results ul{margin:0;padding:0}#search-results li{list-style:none;margin:15px 0}#search-results li .result-title{display:block}#search-results li .result-title a{font-size:22px}#search-results li .result-url{font-size:80%;color:green;display:block}#search-results li .result-modified{color:#7b7b7b}#search-results li .result-modified::after{content:" - "}#search-results li .result-contentType{font-size:11px;color:#888888;font-weight:200;display:inline-block;padding-right:3px;float:left;margin-top:5px}#search-results li .result-download{display:block}#search-results .bottom-total{margin-top:30px}#search-results .bottom-total a{font-size:20px;text-decoration:none}#search-results .bottom-total a:hover{text-decoration:none}#search-results .bottom-total .prev{margin-right:29px}#search-results .bottom-total .next::after{content:">";padding-left:5px}#search-results .bottom-total .prev::before{content:"<";padding-right:5px}