@import "@{bowerPath}/bootstrap/less/variables.less";
@import 'plone-variables';
@import "variables.less";

@import "@{bowerPath}/bootstrap/less/mixins.less";
@import "@{bowerPath}/bootstrap/less/list-group.less";
@import "@{bowerPath}/bootstrap/less/grid.less";

@import (less) "../libs/cropper/dist/cropper.min.css";

@import "logged-in/autotabs";
@import "logged-in/relateditems.less";
@import "logged-in/modals.less";
@import "logged-in/tables.less";
@import "logged-in/upload.less";
@import "logged-in/toolbar.less";
@import "logged-in/editing.less";
@import "logged-in/mosaic.less";
@import "logged-in/tour.less";
@import "logged-in/dashboard.less";
@import "logged-in/onpage.less";
@import "logged-in/control-panel.less";
@import "logged-in/structure.less";
@import "logged-in/fixes.less";
@import "logged-in/icons.less";
@import "logged-in/chat.less";
