/* overridding plone's base less file to not include icons */
@fontsPrefix: '@{staticPath}/fonts/';

@import url("@{mockup-patterns-select2}");
@import url("@{mockup-patterns-pickadate}");
@import url("@{mockup-patterns-autotoc}");
@import url("@{mockup-patterns-modal}");
@import url("@{mockup-patterns-markspeciallinks}");
@import (less) "@{tinymce-default-styles}";

@import "@{bowerPath}bootstrap/less/variables.less";
@icon-font-path: "@{bowerPath}bootstrap/dist/fonts/";

/* tinymce added image styles */
.image-left {
  float: left;
  margin: 0.5em 1em 0.5em 0;
}
.image-right {
  float: right;
  margin: 0.5em;
}
.image-inline {
  float: none;
}

/* icons-off styles to hide icons from
   site setting */
body#visual-portal-wrapper.pat-plone.icons-off .outer-wrapper [class*="contenttype-"]:before {
  display:none;
}

/* error message styles */
body.template-default_error_message #content-core p {
  white-space: pre;
}
