@mockup-patterns-autotoc: '%(mockup-patterns-autotoc.less)s';
@mockup-patterns-filemanager: '%(mockup-patterns-filemanager.less)s';
@mockup-patterns-livesearch: '%(mockup-patterns-livesearch.less)s';
@mockup-patterns-markspeciallinks: '%(mockup-patterns-markspeciallinks.less)s';
@mockup-patterns-modal: '%(mockup-patterns-modal.less)s';
@mockup-patterns-pickadate: '%(mockup-patterns-pickadate.less)s';
@mockup-patterns-querystring: '%(mockup-patterns-querystring.less)s';
@mockup-patterns-recurrence: '%(mockup-patterns-recurrence.less)s';
@mockup-patterns-relateditems: '%(mockup-patterns-relateditems.less)s';
@mockup-patterns-select2: '%(mockup-patterns-select2.less)s';
@mockup-patterns-structure: '%(mockup-patterns-structure.less)s';
@mockup-patterns-tinymce: '%(mockup-patterns-tinymce.less)s';
@mockup-patterns-upload: '%(mockup-patterns-upload.less)s';
@plone-patterns-toolbar: '%(plone-patterns-toolbar.less)s';
@tinymce-default-styles: '%(plone)s/tinymce-styles.css';
@plone-toolbar-submenu-width: 180px;
