@import (less) "../../libs/intro.js/introjs.css";

.introjs-helperNumberLayer {
  line-height: 12px;
  margin-left: 210px;
  left: auto;
  right: 0px;
  top: 6px;
}

.introjs-helperLayer {
  background: rgba(235,235,235,0.0) !important; // lesshint importantRule: false
  z-index: 8888 !important; // lesshint importantRule: false
  display: none;
}

.introjs-overlay {
  background: black;
}

.introjs-tooltip {
  min-width: 300px;
  min-width: 400px;
  .introjs-tooltiptext {
    font-size: 120%;
    margin: 20px 20px 0 20px;
  }
}
